import net.sf.emff.GroupUtil;
import net.sf.emff.Playlist;
import net.sf.emff.Track;
import net.sf.emff.configuration.Configuration;

import net.sf.flashlib.lang.NullPointerException;
import net.sf.flashlib.util.Delegate;



/**
 * Class for an easy musicplayer for Flash (EMFF).<p>
 * Use this class in order to create an easy-to-use musicplayer.<br>
 * The EMFF is licensed under the <a href="http://www.gnu.org/licenses/gpl.html">
 * GNU General Public License</a>. See the following links to get more information
 * about the EMFF:<p>
 * <ul>
 *  <li><a href="http://emff.sourceforge.net/">Official homepage</a></li>
 *  <li><a href="http://sourceforge.net/projects/emff/">Project homepage at sourceforge.net</a></li>
 * </ul>
 * The following projects are used:<p>
 * <ul>
 *  <li><a href="http://sourceforge.net/projects/flashlib/">Flashlib</a></li>
 *  <li><a href="http://mtasc.org/">MTASC</a></li>
 *  <li><a href="http://swfmill.org/">swfmill</a></li>
 * </ul>
 * 
 * @author  Marc Reichelt, <a href="http://www.marcreichelt.de/">http://www.marcreichelt.de/</a>
 * @version 0.0.9
 */
class net.sf.emff.EMFF {
	
	/**
	 * Player is stopped.
	 */
	public static /* const */ var STOPPED : Number = 0;
	
	/**
	 * Player is playing.
	 */
	public static /* const */ var PLAYING : Number = 1;
	
	/**
	 * Player is paused.
	 */
	public static /* const */ var PAUSED : Number = 2;
	
	/**
	 * The configuration.
	 */
	private var configuration : Configuration = null;
	
	/**
	 * The playlist.
	 */
	private var playlist : Playlist = null;
	
	/**
	 * The sound objects.
	 */
	public var sounds : Array = null;
	
	/**
	 * Indicates whether the sound object is loaded or not.
	 */
	private var loaded : Array = null;
	
	/**
	 * The current track index.
	 */
	private var index : Number = 0;
	
	/**
	 * The current track position.
	 */
	private var position : Number = 0;
	
	/**
	 * Status of the player.
	 */
	private var status : Number = STOPPED;
	
	/**
	 * Indicates whether this EMFF is initialized.
	 */
	private var initialized : Boolean = false;
	
	/**
	 * Utility to group multiple EMFF instances.
	 */
	private var groupUtil : GroupUtil = null;
	
	
	
	/**
	 * Creates a new EMFF with the given configuration.
	 * 
	 * @param configuration The configuration.
	 * @throws NullPointerException if the given configuration is <code>null</code>.
	 * @see net.sf.emff.configuration.Configuration
	 */
	public function EMFF(configuration : Configuration) {
		if (configuration == null) {
			throw new NullPointerException();
		}
		this.configuration = configuration;
		
		// create playlist
		playlist = new Playlist(configuration.src, configuration.playlist, configuration.shuffle);
		
		if (playlist.isInitialized()) {
			// playlist loaded directly
			init();
		} else {
			// playlist not loaded yet
			playlist.onInit = Delegate.create(this, init);
		}
		
		if (configuration.shortcuts) {
			// add key listener
			Key.addListener(this);
		}
		
		groupUtil = new GroupUtil(this);
	}
	
	// TODO Docs
	private function init() : Void {
		initialized = true;
		
		// create arrays with initial values
		sounds = new Array();
		loaded = new Array();
		
		var emffSoundMC : MovieClip = _root.createEmptyMovieClip("emffSoundMC", _root.getNextHighestDepth());
		
		for (var i : Number = 0; i < playlist.getLength(); i++) {
			// this is a _dirty_ hack (can you see the dust puppies on the screen?)
			/**
			 * TODO: Check if there is a better solution (e.g. with one sound object)
			 */
			var soundMC : MovieClip = emffSoundMC.createEmptyMovieClip("soundMC" + i, emffSoundMC.getNextHighestDepth());
			
			var sound : Sound = new Sound(soundMC);
			sound.setVolume(configuration.volume);
			sound.setPan(configuration.balance);
			sound.onSoundComplete = Delegate.create(this, soundComplete);
			sound.onLoad = Delegate.create(this, soundLoad);
			sounds.push(sound);
			
			loaded.push(false);
		}
		
		if (configuration.autostart) {
			play();
		} else if (configuration.autoload) {
			load();
		}
	}
	
	/**
	 * Go to previous track.
	 */
	public function previous() : Void {
		if (!initialized) {
			return;
		}
		
		Sound(sounds[index]).stop();
		
		// decrease index
		index--;
		if (index < 0) {
			if (configuration.repeat) {
				index = playlist.getLength() - 1;
			} else {
				index++;
			}
		}
		
		toNewTrack();
	}
	
	/**
	 * Go to next track.
	 */
	public function next() : Void {
		if (!initialized) {
			return;
		}
		
		Sound(sounds[index]).stop();
		
		// increase index
		index++;
		if (index >= playlist.getLength()) {
			if (configuration.repeat) {
				index = 0;
			} else {
				index--;
				status = STOPPED;
				return;
			}
		}
		
		toNewTrack();
	}
	
	/**
	 * Stop immediately.
	 */
	public function stop() : Void {
		if (!initialized) {
			return;
		}
		
		if (status != STOPPED) {
			status = STOPPED;
			Sound(sounds[index]).stop();
		}
	}
	
	/**
	 * Pause.
	 */
	public function pause() : Void {
		if (!initialized) {
			return;
		}
		
		if (status == PLAYING) {
			status = PAUSED;
			position = Sound(sounds[index]).position;
			Sound(sounds[index]).stop();
		}
	}
	
	/**
	 * Play the current track.
	 */
	public function play() : Void {
		if (!initialized) {
			return;
		}
		
		// notify all other EMFF instance in a group that this instance is active right now
		groupUtil.setActive();
		
		switch (status) {
		case STOPPED:
			status = PLAYING;
			if (!loaded[index]) {
				loaded[index] = true;
				
				Sound(sounds[index]).loadSound(playlist.getTrack(index).getLocation(), true);
				Sound(sounds[index]).start();
			} else {
				Sound(sounds[index]).start();
			}
			break;
		case PLAYING:
			break;
		case PAUSED:
			status = PLAYING;
			Sound(sounds[index]).start(Math.round(position / 1000));
			break;
		}
		
		// again an ugly hack
		Sound(sounds[index]).setVolume(configuration.volume);
		Sound(sounds[index]).setPan(configuration.balance);
	}
	
	/**
	 * Loads the current track without playing it.
	 */
	private function load() : Void {
		if (!loaded[index]) {
			loaded[index] = true;
			
			Sound(sounds[index]).loadSound(playlist.getTrack(index).getLocation(), true);
			Sound(sounds[index]).stop();
		}
	}
	
	/**
	 * Get volume.
	 * 
	 * @return The current volume, a number between <code>0</code> and <code>100</code>.
	 * @see net.sf.emff.Player#getVolume()
	 */
	public function getVolume() : Number {
		return configuration.volume;
	}
	
	/**
	 * Get balance.
	 * 
	 * @return The current balance, a number between <code>-100</code> and <code>100</code>.
	 * @see net.sf.emff.Player#getBalance()
	 */
	public function getBalance() : Number {
		return configuration.balance;
	}
	
	/**
	 * Get playlist.
	 * 
	 * @return The playlist, an instance of <code>Playlist</code>.
	 * @see net.sf.emff.Playlist
	 */
	public function getPlaylist() : Playlist {
		return playlist;
	}
	
	/**
	 * Get loading progress.
	 * 
	 * @return The loading progress, a number between <code>0</code> (for 0 percent) and
	 *         <code>1</code> (for 100 %).
	 */
	public function getLoadingProgress() : Number {
		if (!initialized) {
			return 0;
		}
		
		if (
			isNaN(Sound(sounds[index]).getBytesLoaded()) ||
			isNaN(Sound(sounds[index]).getBytesTotal())
		) {
			return 0;
		} else {
			return Sound(sounds[index]).getBytesLoaded() / Sound(sounds[index]).getBytesTotal();
		}
	}
	
	/**
	 * Get position (in milliseconds).
	 * 
	 * @return Position in the active track (in milliseconds).
	 */
	public function getPosition() : Number {
		if (!initialized) {
			return 0;
		}
		
		switch (status) {
		case PLAYING:
			return Sound(sounds[index]).position;
		case PAUSED:
			return position;
		default:
			return 0;
		}
	}
	
	/**
	 * Get playling progress.
	 * 
	 * @return The playing progress, a number between <code>0</code> (for 0 percent) and
	 *         <code>1</code> (for 100 %).
	 */
	public function getPlayingProgress() : Number {
		if (!initialized) {
			return 0;
		}
		
		var load : Number = getLoadingProgress();
		
		if (load == 0) {
			return 0;
		}
		
		var playedmsec : Number = Sound(sounds[index]).position;
		var totalmsec : Number = Sound(sounds[index]).duration / getLoadingProgress();
		
		switch (status) {
		case PLAYING:
			return playedmsec / totalmsec;
		case PAUSED:
			return position / totalmsec;
		default:
			return 0;
		}
	}
	
	/**
	 * Set volume.
	 * 
	 * @param volume The volume (a number between <code>0</code> and <code>100</code>).
	 */
	public function setVolume(volume : Number) : Void {
		if (!initialized) {
			return;
		}
		
		Sound(sounds[index]).setVolume(volume);
		configuration.volume = Sound(sounds[index]).getVolume();
	}
	
	/**
	 * Set balance.
	 * 
	 * @param balance The balance (a number between <code>-100</code> and <code>100</code>).
	 */
	public function setBalance(balance : Number) : Void {
		if (!initialized) {
			return;
		}
		
		Sound(sounds[index]).setPan(balance);
		configuration.balance = Sound(sounds[index]).getPan();
	}
	
	/**
	 * Get status.
	 * 
	 * @return Status (number between 0 and 2, where 0 stands for STOPPED, 1 for
	 *          PLAYING and 2 for PAUSED.
	 */
	public function getStatus() : Number {
		return status;
	}
	
	/**
	 * Get the index for the active track.
	 * 
	 * @return Index of the active track.
	 */
	public function getActiveIndex() : Number {
		return index;
	}
	
	/**
	 * Get the active track.
	 * 
	 * @return Active track.
	 */
	public function getActiveTrack() : Track {
		if (!initialized) {
			return new Track(null);
		}
		
		return playlist.getTrack(index);
	}
	
	/**
	 * Set the active index.
	 * 
	 * @param index The index of the new track (the EMFF will jump to that one).
	 */
	public function setActiveIndex(index : Number) : Void {
		if (!initialized) {
			return;
		}
		
		if (index != null && index >= 0 && index < playlist.getLength()) {
			Sound(sounds[this.index]).stop();
			this.index = index;
			toNewTrack();
		}
	}
	
	/**
	 * Jumps to a relative position of this track.
	 * Only works if the status of the player is PAUSED.
	 * 
	 * @param The relative position (number between 0 and 1) where the track
	 *         should jump to.
	 */
	public function setRelativePosition(relativePosition : Number) : Void {
		if (!initialized) {
			return;
		}
		
		if (relativePosition >= 0 && relativePosition <= 1 && status == PAUSED) {
			// calculate the offset to jump to
			position = relativePosition * Sound(sounds[index]).duration / getLoadingProgress();
		}
	}
	
	/**
	 * Gets the configuration.
	 * 
	 * @return Configuration object for this EMFF.
	 */
	public function getConfiguration() : Configuration {
		return configuration;
	}
	
	
	
	private function toNewTrack() : Void {
		if (status != STOPPED) {
			status = STOPPED;
			play();
		}
	}
	
	private function soundComplete() : Void {
		next();
	}
	
	private function soundLoad(success : Boolean) : Void {
		if (!success) {
			next();
		}
	}
	
	private function onKeyUp() : Void {
		switch (Key.getCode()) {
		case "Z".charCodeAt(0):
			previous();
			break;
		case "X".charCodeAt(0):
			play();
			break;
		case "C".charCodeAt(0):
			pause();
			break;
		case "V".charCodeAt(0):
			stop();
			break;
		case "B".charCodeAt(0):
			next();
			break;
		}
	}
	
}
