/**
 * A track.<br>
 * The properties of a track are created according to the
 * <a href="http://www.xspf.org/">XSPF</a> standard.
 * 
 * @version  0.0.2
 * @author   Marc Reichelt
 */
class net.sf.emff.Track {
	
	/**
	 * Location (URL).
	 */
	private var location : String = "default.mp3";
	
	/**
	 * Title.
	 */
	private var title : String = "";
	
	/**
	 * Creator.
	 */
	private var creator : String = "";
	
	/**
	 * Annotation.
	 */
	private var annotation : String = "";
	
	/**
	 * URL to more information.
	 */
	private var info : String = "";
	
	/**
	 * URL to an image.
	 */
	private var image : String = "";
	
	/**
	 * Album.
	 */
	private var album : String = "";
	
	/**
	 * Track number of album.
	 */
	private var trackNum : Number = null;
	
	/**
	 * Duration of this track (in milliseconds).
	 */
	private var duration : Number = null;
	
	
	
	/**
	 * Creates a new Track with the given location.
	 * 
	 * @param  location    Location (URL).
	 */
	public function Track(location : String) {
		setLocation(location);
	}
	
	
	
	/**
	 * Set the location.
	 * 
	 * @param  location  Location (URL).
	 */
	public function setLocation(location : String) : Void {
		if (location != null) {
			this.location = location;
		}
	}
	
	/**
	 * Set the title.
	 * 
	 * @param  title  Title.
	 */
	public function setTitle(title : String) : Void {
		if (title != null) {
			this.title = title;
		}
	}
	
	/**
	 * Set the creator.
	 * 
	 * @param  creator  Creator.
	 */
	public function setCreator(creator : String) : Void {
		if (creator != null) {
			this.creator = creator;
		}
	}
	
	/**
	 * Set annotation.
	 * 
	 * @param  annotation  Annotation.
	 */
	public function setAnnotation(annotation : String) : Void {
		if (annotation != null) {
			this.annotation = annotation;
		}
	}
	
	/**
	 * Set URL to more information.
	 * 
	 * @param  info  URL to more information.
	 */
	public function setInfo(info : String) : Void {
		if (info != null) {
			this.info = info;
		}
	}
	
	/**
	 * Set image.
	 * 
	 * @param  image  URL to an image.
	 */
	public function setImage(image : String) : Void {
		if (image != null) {
			this.image = image;
		}
	}
	
	/**
	 * Set album.
	 * 
	 * @param  album  Album.
	 */
	public function setAlbum(album : String) : Void {
		if (album != null) {
			this.album = album;
		}
	}
	
	/**
	 * Set track number of album.
	 * 
	 * @param  trackNum  Positive track number of album.
	 */
	public function setTrackNum(trackNum : Number) : Void {
		if (trackNum == null || trackNum >= 0) {
			this.trackNum = trackNum;
		}
	}
	
	/**
	 * Set duration.
	 * 
	 * @param  duration  Duration of this track (in milliseconds), must be positive.
	 */
	public function setDuration(duration : Number) : Void {
		if (duration == null || duration >= 0) {
			this.duration = duration;
		}
	}
	
	
	
	/**
	 * Get location URL.
	 * 
	 * @return  Location (URL).
	 */
	public function getLocation() : String {
		return location;
	}
	
	/**
	 * Get title.
	 * 
	 * @return  Title.
	 */
	public function getTitle() : String {
		return title;
	}
	
	/**
	 * Get creator.
	 * 
	 * @return  Creator.
	 */
	public function getCreator() : String {
		return creator;
	}
	
	/**
	 * Get annotation.
	 * 
	 * @return  Annotation.
	 */
	public function getAnnotation() : String {
		return annotation;
	}
	
	/**
	 * Get URL to information.
	 * 
	 * @return  URL to more information.
	 */
	public function getInfo() : String {
		return info;
	}
	
	/**
	 * Get image.
	 * 
	 * @return  URL to an image.
	 */
	public function getImage() : String {
		return image;
	}
	
	/**
	 * Get album.
	 * 
	 * @return  Album.
	 */
	public function getAlbum() : String {
		return album;
	}
	
	/**
	 * Get track number of album.
	 * 
	 * @return  Track number of album.
	 */
	public function getTrackNum() : Number {
		return trackNum;
	}
	
	/**
	 * Get duration.
	 * 
	 * @return  Duration (in milliseconds).
	 */
	public function getDuration() : Number {
		return duration;
	}
	
	/**
	 * Get a string representation for this track.
	 * 
	 * @result String representation for this track.
	 */
	public function toString() : String {
		var result : String =
			"Track { "
			+ "location='" + location + "', "
			+ "title='" + title + "', "
			+ "creator='" + creator + "', "
			+ "annotation='" + annotation + "', "
			+ "info='" + info + "', "
			+ "image='" + image + "', "
			+ "album='" + album + "', "
			+ "trackNum=" + trackNum + ", "
			+ "duration=" + duration
			+ " }";
		
		return result;
	}
	
}
