import net.sf.flashlib.util.Delegate;

import net.sf.flashlib.util.logging.Logger;
import net.sf.flashlib.util.logging.LogMessage;

import net.sf.flashlib.lang.NullPointerException;



/**
 * A logger that sends all messages to a HTTP server.<br>
 * There a CGI script could listen and save all messages to a simple text file.
 * 
 * @version 0.1.1
 * @author  Marc Reichelt
 */
class net.sf.flashlib.util.logging.HTTPLogger extends Logger {
	
	public static /* const */ var MAX_TEXT : Number = 1024 * 1024 * 1024;
	
	private var url : String;
	private var username : String;
	private var password : String;
	private var method : String;
	private var loadVars : LoadVars;
	private var isBusy : Boolean = false;
	private var buffer : Array = new Array();
	
	
	
	/**
	 * Creates a new HTTP logger with the given parameters.
	 * 
	 * @param  url            The URL this logger sends the messages to. The default is "http://127.0.0.1/httplogger/log/".
	 * @param  username       HTTPLogger will send this parameter as "username" to the target CGI script. The default is "root".
	 * @param  password       HTTPLogger will send this parameter as "password" to the target CGI script. The default is "" (no password).<br>
	 *                         Note that all data is sent unencrypted by default, which is unsecure!
	 * @param  method         The method to use, could be "POST" or "GET". "POST" is the default.
	 */
	public function HTTPLogger(url : String, username : String, password : String, method : String) {
		
		// get parameters, set default values if the parameter isn't specified or is incorrect
		this.url = (url == null) ? "http://127.0.0.1/httplogger/log/" : url;
		this.username = (username == null) ? "root" : username;
		this.password = (password == null) ? "" : password;
		this.method = (method == null && method != "POST" && method != "GET") ? "POST" : method;
		
		// create LoadVars object and apply onLoad method
		createNewLoadVars();
		
	}
	
	
	
	/**
	 * Log a specific log message.
	 * 
	 * @param  message  The log message, an instance of LogMessage.
	 * @see    net.sf.flashlib.util.logging.Logger#log()
	 */
	public function log(message : LogMessage) : Void {
		
		if (message == null) {
			throw new NullPointerException("The message must not be null.");
		}
		
		// check if the level value of the message is lower than or equal to our level value
		if (message.getLevel().getValue() >= this.getLevel().getValue()) {
			
			// add message to buffer
			buffer.push(message);
			
			sendToServer();
		}
		
	}
	
	/**
	 * Send the next queued message to the server.
	 */
	private function sendToServer() : Void {
		if (!isBusy) {
			isBusy = true;
			loadVars.username = username;
			loadVars.password = password;
			loadVars.text = "";
			loadVars.text += LogMessage(buffer.shift()).toString();
			
			// send data
			loadVars.sendAndLoad(url, loadVars, method);
			
		}
	}
	
	/**
	 * This function is called by the LoadVars object.
	 * 
	 * @param  success  true if no errors occured during transfer, otherwise false.
	 */
	private function onLoad(success : Boolean) : Void {
		
		if (!success) {
			// a failure occured
			// TODO: throw own exception
		} else {
			// no failure
		}
		
		createNewLoadVars();
		isBusy = false;
		
		if (buffer.length > 0) {
			sendToServer();
		}
	}
	
	private function createNewLoadVars() : Void {
		loadVars = new LoadVars();
		loadVars.onLoad = Delegate.create(this, onLoad);
	}
	
}
