import net.sf.flashlib.lang.NullPointerException;

/**
 * Level for loggers.
 * 
 * @version 0.1.0
 * @author  Marc Reichelt
 */
class net.sf.flashlib.util.logging.Level {
	
	/**
	 * OFF is a special level that can be used to turn off logging. This level is initialized to Number.MAX_VALUE.
	 */
	public static /* const */ var OFF : Level = new Level("OFF", Number.MAX_VALUE);
	
	/**
	 * SEVERE is a message level indicating a serious failure. This level is initialized to 1000.
	 */
	public static /* const */ var SEVERE : Level = new Level("SEVERE", 1000);
	
	/**
	 * WARNING is a message level indicating a potential problem. This level is initialized to 900.
	 */
	public static /* const */ var WARNING : Level = new Level("WARNING", 900);
	
	/**
	 * INFO is a message level for informational messages. This level is initialized to 800.
	 */
	public static /* const */ var INFO : Level = new Level("INFO", 800);
	
	/**
	 * CONFIG is a message level for static configuration messages. This level is initialized to 700.
	 */
	public static /* const */ var CONFIG : Level = new Level("CONFIG", 700);
	
	/**
	 * FINE messages might include things like minor (recoverable) failures. Issues indicating potential performance problems are also worth logging as FINE. This level is initialized to 500.
	 */
	public static /* const */ var FINE : Level = new Level("FINE", 500);
	
	/**
	 * FINER indicates a fairly detailed tracing message. By default logging calls for entering, returning, or throwing an exception (formerly Error in ActionScript 2.0) are traced at this level. This level is initialized to 400.
	 */
	public static /* const */ var FINER : Level = new Level("FINER", 400);
	
	/**
	 * FINEST indicates a highly detailed tracing message. This level is initialized to 300.
	 */
	public static /* const */ var FINEST : Level = new Level("FINEST", 300);
	
	/**
	 * ALL indicates that all messages should be logged. This level is initialized to Number.MIN_VALUE.
	 */
	public static /* const */ var ALL : Level = new Level("ALL", Number.MIN_VALUE);
	
	/**
	 * Name of this level, e.g. "INFO".
	 */
	private var name : String;
	
	/**
	 * Value of this level - higher numbers are more important than lower ones.
	 */
	private var value : Number;
	
	
	
	/**
	 * Creates a new level for logging.
	 * 
	 * @param  name   Name of level.
	 * @param  value  Value of level.
	 */
	public function Level(name : String, value : Number) {
		if (name == null) {
			throw new NullPointerException("name of a Level must not be null.");
		}
		if (value == null) {
			throw new NullPointerException("value of a Level must not be null.");
		}
		
		this.name = name;
		this.value = value;
	}
	
	
	
	/**
	 * Gets the name of this level.
	 * 
	 * @return Name of this level.
	 */
	public function getName() : String {
		return name;
	}
	
	/**
	 * Gets the value of this level.
	 * 
	 * @return Value of this level.
	 */
	public function getValue() : Number {
		return value;
	}
	
}
