import net.sf.flashlib.util.logging.Level;

/**
 * A simple log message.
 * 
 * @version 0.1.0
 * @author  Marc Reichelt
 */
class net.sf.flashlib.util.logging.LogMessage {
	
	/**
	 * The level.
	 */
	private var level : Level;
	
	/**
	 * The message.
	 */
	private var message : String;
	
	/**
	 * The date.
	 */
	private var date : Date;
	
	/**
	 * String representation of this log message.
	 */
	private var str : String;
	
	/**
	 * RFC 2822 formatted days.
	 */
	private static /*const */ var days : Array = new Array(
		"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
	);
	
	/**
	 * RFC 2822 formatted months.
	 */
	private static /*const */ var months : Array = new Array(
		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
	);
	
	
	
	/**
	 * Creates a new log message with the given parameters.
	 * 
	 * @param  level    The level of this message, default is Level.INFO.
	 * @param  message  The message, default is "".
	 */
	public function LogMessage(level : Level, message : String) {
		
		// save parameters, set default values
		this.level = (level == null) ? Level.INFO : level;
		this.message = (message == null) ? "" : message;
		
		// get current date
		this.date = new Date();
		
		// build string
		this.str = days[date.getUTCDay()] + ", " + fillWithZeroes(String(date.getUTCDate()), 2) + " " +
		           months[date.getUTCMonth()] + " " + date.getUTCFullYear() + " " + fillWithZeroes(String(date.getUTCHours()), 2) + ":" +
		           fillWithZeroes(String(date.getUTCMinutes()), 2) + ":" + fillWithZeroes(String(date.getUTCSeconds()), 2) + " " +
		           getFormattedTimezoneOffset() + " - " + level.getName() + " - " + message + "\n";
		
	}
	
	
	
	/**
	 * Get level.
	 * 
	 * @return  The level.
	 */
	public function getLevel() : Level {
		return level;
	}
	
	/**
	 * Get message.
	 * 
	 * @return  The message.
	 */
	public function getMessage() : String {
		return message;
	}
	
	/**
	 * Get date.
	 * 
	 * @return  The date.
	 */
	public function getDate() : Date {
		return date;
	}
	
	/**
	 * Get string representation of this log message.
	 * 
	 * @return  String representation of this log message.
	 */
	public function toString() : String {
		return str;
	}
	
	/**
	 * Function that fills up a specific String with zeroes from left side.
	 * 
	 * @param   input   Input string.
	 * @param   length  Length that the final String will have.
	 * @return  Input string, filled with zeroes from the left.
	 */
	private function fillWithZeroes(input : String, length : Number) : String {
		var result : String = new String(input);
		for (var rest : Number = length - input.length; rest > 0; rest--) {
			result = "0" + result;
		}
		return result;
	}
	
	
	/**
	 * Returns the formatted timezone offset.
	 * 
	 * @return  Formatted timezone offset.
	 */
	private function getFormattedTimezoneOffset() : String {
		var timezoneOffset : Number = date.getTimezoneOffset();
		var result : String = "";
		
		if (timezoneOffset < 0) {
			result += "+";
			result += fillWithZeroes(String(Math.floor(-timezoneOffset / 60)), 2);
		} else {
			result += "-";
			result += fillWithZeroes(String(Math.floor(timezoneOffset / 60)), 2);
		}
		
		result += fillWithZeroes(String(timezoneOffset % 60), 2);
		
		return result;
	}
	
}
