import net.sf.flashlib.util.Observable;

import net.sf.flashlib.util.logging.Level;
import net.sf.flashlib.util.logging.LogMessage;

/**
 * A basic logger.
 * 
 * @version 0.1.1
 * @author  Marc Reichelt
 */
/* abstract */ class net.sf.flashlib.util.logging.Logger extends Observable {
	
	/**
	 * Level of this logger - default is Level.ALL (log all messages).
	 */
	private var level : Level = Level.ALL;
	
	
	
	/**
	 * Log a specific log message.
	 * 
	 * @param  message  The log message, an instance of LogMessage.
	 */
	public /* abstract */ function log(message : LogMessage) : Void {
		// empty method body, this method should be declared as abstract in future
	}
	
	/**
	 * Log a message indicating a serious failure.
	 * 
	 * @param  message  The message.
	 */
	public function severe(message : String) : Void {
		log(new LogMessage(Level.SEVERE, message));
	}
	
	/**
	 * Log a message indicating a potential problem.
	 * 
	 * @param  message  The message.
	 */
	public function warning(message : String) : Void {
		log(new LogMessage(Level.WARNING, message));
	}
	
	/**
	 * Log an informational message.
	 * 
	 * @param  message  The message.
	 */
	public function info(message : String) : Void {
		log(new LogMessage(Level.INFO, message));
	}
	
	/**
	 * Log a message indicating a static configuration.
	 * 
	 * @param  message  The message.
	 */
	public function config(message : String) : Void {
		log(new LogMessage(Level.CONFIG, message));
	}
	
	/**
	 * Log a message indicating things like minor (recoverable) failures.
	 * 
	 * @param  message  The message.
	 */
	public function fine(message : String) : Void {
		log(new LogMessage(Level.FINE, message));
	}
	
	/**
	 * Log a fairly detailed tracing message.
	 * 
	 * @param  message  The message.
	 */
	public function finer(message : String) : Void {
		log(new LogMessage(Level.FINER, message));
	}
	
	/**
	 * Log a highly detailed tracing message.
	 * 
	 * @param  message  The message.
	 */
	public function finest(message : String) : Void {
		log(new LogMessage(Level.FINEST, message));
	}
	
	/**
	 * Set level of this logger.
	 * 
	 * @param  level  The level.
	 */
	public function setLevel(level : Level) : Void {
		if (level != null) {
			this.level = level;
		}
	}
	
	/**
	 * Get level of this logger.
	 * 
	 * @return  The level.
	 */
	public function getLevel() : Level {
		return level;
	}
	
}
