import net.sf.flashlib.util.logging.Logger;
import net.sf.flashlib.util.logging.LogMessage;

import net.sf.flashlib.lang.NullPointerException;



/**
 * A logger that sends all messages to a text field.
 * 
 * @version 0.2.0
 * @author  Marc Reichelt
 */
class net.sf.flashlib.util.logging.TextFieldLogger extends Logger {
	
	/**
	 * The text field.
	 */
	private var textField : TextField;
	
	/**
	 * If true, the text field will automagically scroll to the newest message.
	 */
	private var autoscroll : Boolean;
	
	
	
	/**
	 * Creates a new logger that prints out its messages into a text field.
	 * 
	 * @param  textField  The TextField object to print out the messages.
	 * @param  autoscroll If true, the text field will automagically scroll to the newest message. By default, autoscroll is true.
	 */
	public function TextFieldLogger(textField : TextField, autoscroll : Boolean) {
		
		if (textField == null) {
			throw new NullPointerException("textField must not be null.");
		}
		
		this.textField  = textField;
		this.autoscroll = (autoscroll == null) ? true : autoscroll;
		
		textField.wordWrap = true;
	}
	
	
	
	/**
	 * Log a specific log message.
	 * 
	 * @param  message  The log message, an instance of LogMessage.
	 * @see    net.sf.flashlib.util.logging.Logger#log()
	 */
	public function log(message : LogMessage) : Void {
		
		if (message == null) {
			throw new NullPointerException("The message must not be null.");
		}
		
		// check if the level value of the message is lower than or equal to our level value
		if (message.getLevel().getValue() >= this.getLevel().getValue()) {
			
			setChanged();
			notifyObservers();
			
			// print out message
			textField.text += message.toString();
			
			// scroll to the newest message if required
			if (autoscroll) {
				textField.scroll = textField.maxscroll;
			}
			
		}
		
	}
	
}
