import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;


public class UpdateCodeGenerator {

	private static final File OUTPUT_FILE = new File("codegenerator" + File.separator + "skins.js");
	
	
	
	/**
	 * Reads all SKIN files and updates the "skins.js" file.
	 */
	public static void main(String[] args) {
		Hashtable<String, File> skins = CreateSkin.getAllSkins();
		ArrayList<String> skinNames = new ArrayList<String>(skins.keySet());
		Collections.sort(skinNames);
		
		try {
			
			// open output file
			FileWriter writer = new FileWriter(OUTPUT_FILE);
			
			writer.write("var skins = new Array();\n\n");
			
			for (String skinName : skinNames) {
				Hashtable<String, String> info = getInfo(skins.get(skinName));
				
				if (info.get("exclude").equals("yes")) {
					// this skin is excluded from the code generator
					continue;
				}
				
				writer.write("skins.push(new Skin(");
				
				writer.write("\"" + addSlashes(skinName) + "\", ");
				writer.write("\"" + addSlashes(info.get("author")) + "\", ");
				writer.write("\"" + addSlashes(info.get("authorUrl")) + "\", ");
				writer.write("\"" + addSlashes(info.get("width")) + "\", ");
				writer.write("\"" + addSlashes(info.get("height")) + "\", ");
				writer.write("\"" + addSlashes(info.get("credits")) + "\"");
				
				writer.write("));\n");
			}
			
			writer.close();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Parses a SKIN file.
	 * 
	 * @param skinPath The path to the skin directory where the SKIN file is in.
	 * @return All parsed data for the appropriate SKIN file.
	 */
	private static Hashtable<String, String> getInfo(File skinPath) {
		Hashtable<String, String> info = new Hashtable<String, String>();
		
		info.put("exclude", "no");
		info.put("author", "");
		info.put("authorUrl", "");
		info.put("width", "");
		info.put("height", "");
		info.put("credits", "");
		
		try {
			// parse info file
			BufferedReader reader = new BufferedReader(new FileReader(skinPath.getCanonicalPath() + File.separator + "SKIN"));
			String line;
			while ((line = reader.readLine()) != null) {
				
				line = line.trim();
				
				if (line.length() != 0 && !line.startsWith("#")) {
					String[] data = line.split("\\=", 2);
					if (data.length == 2) {
						info.put(data[0], data[1]);
					}
				}
			}
		} catch (Exception e) {
			
		}
		
		return info;
	}
	
	public static String addSlashes(String str) {
		str = str.replace("\\", "\\\\");
		str = str.replace("\"", "\\\"");
		
		return str;
	}

}
